import { defineStore } from "pinia"
import { computed, reactive, ref } from "vue"
import { script } from "./shared/script"
import type { GameObject, Interaction, InteractionQueueItem } from "./shared/script/types"
import { trpcClient } from "./trpc"
import type { GameEvent } from "./shared/gameEvents"
import { getInteractionQueueItemId } from "./shared/util"

export const useGame = defineStore("gameState", () => {
    const currentRoomId = ref(script.roomsById.values().next().value!.id)
    const interactionQueue = reactive(new Map<string, InteractionQueueItem>())
    const currentInteraction = ref<null | Interaction>(null)
    const currentInteractionId = computed(() =>
        currentInteraction.value === null ? null : getInteractionQueueItemId(currentInteraction.value))

    const sortedInteractionQueue = computed(() => [...interactionQueue.values()].sort((a, b) => b.votes - a.votes))

    const currentRoom = computed(() => script.roomsById.get(currentRoomId.value)!)

    return {
        currentRoomId,
        interactionQueue,
        sortedInteractionQueue,
        currentRoom,
        allObjectsById: computed(() => {
            const map = new Map<string, GameObject>()
            currentRoom.value.initialObjects.forEach(o => map.set(o.id, o))
            currentRoom.value.hiddenObjects.forEach(o => map.set(o.id, o))
            return map
        }),
        currentInteraction,
        currentInteractionId,
        voteForInteraction(interaction: Interaction) {
            if (currentInteractionId.value === getInteractionQueueItemId(interaction)) return
            if (currentInteractionId.value !== null) trpcClient.player.removeInteractionVote.mutate({ queueItemId: currentInteractionId.value })
            currentInteraction.value = interaction
            trpcClient.player.voteForInteraction.mutate({ interaction })
        },
        revokeCurrentInteractionVote() {
            if (currentInteractionId.value === null) return
            trpcClient.player.removeInteractionVote.mutate({ queueItemId: currentInteractionId.value })
            currentInteraction.value = null
        },
        switchRoom(roomId: string) {
            trpcClient.director.switchRoom.mutate({ roomId })
        },
        removeInteractionQueueItem(id: string) {
            trpcClient.director.removeInteractionQueueItem.mutate({ id })
        },
        handleGameEvent(event: GameEvent) {
            console.log(event)
            switch (event.type) {
                case "room-changed":
                    interactionQueue.clear()
                    currentRoomId.value = event.roomId
                    currentInteraction.value = null
                    break

                case "interaction-queued":
                    interactionQueue.set(event.item.id, event.item)
                    break

                case "interaction-votes-changed":
                    if (event.votes <= 0) {
                        interactionQueue.delete(event.id)
                        if (currentInteractionId.value === event.id) currentInteraction.value = null
                    } else {
                        interactionQueue.get(event.id)!.votes = event.votes
                    }

                    break
            }
        }
    }
})